Package: GOplot
Type: Package
Title: Visualization of Functional Analysis Data
Version: 1.0.2
Date: 2016-03-30
Authors@R: c(
    person("Wencke", "Walter", , email = "wencke.walter@arcor.de", role = c("aut", "cre")),
    person("Fatima", "Sanchez-Cabo", , role = "aut")
    )
URL: https://github.com/wencke/wencke.github.io
BugReports: https://github.com/wencke/wencke.github.io/issues
Description: Implementation of multilayered visualizations for enhanced
    graphical representation of functional analysis data. It combines and integrates
    omics data derived from expression and functional annotation enrichment
    analyses. Its plotting functions have been developed with an hierarchical
    structure in mind: starting from a general overview to identify the most
    enriched categories (modified bar plot, bubble plot) to a more detailed one
    displaying different types of relevant information for the molecules in a given
    set of categories (circle plot, chord plot, cluster plot, Venn diagram, heatmap).
Depends: ggplot2 (>= 2.0.0), ggdendro (>= 0.1-17), gridExtra (>=
        2.0.0), RColorBrewer (>= 1.1.2), R (>= 3.2.3)
License: GPL-2
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
LazyData: TRUE
RoxygenNote: 5.0.1
NeedsCompilation: no
Packaged: 2016-03-30 08:24:21 UTC; BioinfoNerd
Author: Wencke Walter [aut, cre],
  Fatima Sanchez-Cabo [aut]
Maintainer: Wencke Walter <wencke.walter@arcor.de>
Repository: CRAN
Date/Publication: 2016-03-30 20:35:02
Built: R 4.6.0; ; 2026-04-28 04:28:19 UTC; unix
