# Generated by roxygen2: do not edit by hand

export(GOBar)
export(GOBubble)
export(GOChord)
export(GOCircle)
export(GOCluster)
export(GOHeat)
export(GOVenn)
export(chord_dat)
export(circle_dat)
export(reduce_overlap)
import(RColorBrewer)
import(ggdendro)
import(ggplot2)
import(grDevices)
import(graphics)
import(gridExtra)
import(stats)
