# GOplot

Despite the plethora of methods available for the functional analysis of omics data, obtaining comprehensive- yet detailed understanding of the results remains challenging. GOplot takes the output of any general enrichment analysis and generates plots at different levels of detail: from a general overview to identify the most enriched categories (bar plot, bubble plot) to a more detailed view displaying different types of information for molecules in a given set of categories (circle plot, chord plot, cluster plot). The package provides a deeper insight into omics data and allows scientists to generate insightful plots with only a few lines of code to easily communicate the findings. 

## Installation

GOplot is available via CRAN: http://cran.r-project.org/web/packages/GOplot

* the latest released version: `install.packages("GOplot")`
* the latest development version: `install_github("wencke/wencke.github.io")`

## Available functions

For preprocessing: circle_dat(), chord_dat() and reduce_overlap()

For plotting: GOBubble(), GOBar(), GOChord(), GOCluster(), GOCircle(), GOVenn(), GOHeat()

A manual can be found on the website https://wencke.github.io/
